{
 "cells": [
  {
   "cell_type": "code",
   "execution_count": null,
   "id": "1f040f7f-2f77-41f1-a734-94d4ae56ec8e",
   "metadata": {},
   "outputs": [],
   "source": [
    "! pip install prance mistral_common openapi-spec-validator pyaudio\n",
    "! pip install --no-build-isolation --force-reinstall \\\n",
    "    \"boto3>=1.28.57\" \\\n",
    "    \"awscli>=1.29.57\" \\\n",
    "    \"botocore>=1.31.57\""
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "id": "55802dfe-0c00-4016-8e9c-1180c102ffcb",
   "metadata": {},
   "outputs": [],
   "source": [
    "\n",
    "import prance\n",
    "from typing import List\n",
    "from mistral_common.tokens.tokenizers.mistral import MistralTokenizer\n",
    "from mistral_common.protocol.instruct.request import ChatCompletionRequest\n",
    "from mistral_common.protocol.instruct.tool_calls import Function, Tool\n",
    "from mistral_common.protocol.instruct.messages import UserMessage\n",
    "import json\n",
    "import requests\n",
    "import functools\n",
    "import boto3\n",
    "import json\n",
    "import pyaudio\n",
    "import wave\n",
    "import os\n",
    "#bedrock = boto3.client(service_name=\"bedrock-runtime\", region_name='us-east-1')\n",
    "bedrock = boto3.client(service_name=\"bedrock-runtime\")\n"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "id": "314a15f5-080d-4a7a-9bed-c1f63421f161",
   "metadata": {},
   "outputs": [],
   "source": [
    "def get_user_messages(queries: List[str]) -> List[UserMessage]:\n",
    "    user_messages=[]\n",
    "    for query in queries:\n",
    "        user_message = UserMessage(content=query)\n",
    "        user_messages.append(user_message)\n",
    "    return user_messages"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "id": "7416b4a5-5272-41b1-be49-28c9fbe16c74",
   "metadata": {},
   "outputs": [],
   "source": [
    "def generate_tools(objs, function_end_point)-> List[Tool]:\n",
    "    params = ['operationId', 'description',  'parameters']\n",
    "\n",
    "    parser = prance.ResolvingParser(function_end_point, backend='openapi-spec-validator')\n",
    "    spec = parser.specification\n",
    "    user_tools = []\n",
    "    for obj in objs:\n",
    "        resource, field = obj\n",
    "        path = '/' + resource + '/{' + field + '}'\n",
    "        function_name=spec['paths'][path]['get'][params[0]]\n",
    "        function_description=spec['paths'][path]['get'][params[1]]\n",
    "        function_parameters=spec['paths'][path]['get'][params[2]]\n",
    "        func_parameters = {\n",
    "        \"type\": \"object\",\n",
    "        \"properties\": {\n",
    "            function_parameters[0]['name']: {\n",
    "                \"type\": function_parameters[0]['schema']['type'],\n",
    "                \"description\": function_parameters[0]['description']\n",
    "            }\n",
    "        },\n",
    "        \"required\": [function_parameters[0]['name']]\n",
    "    }\n",
    "        user_function= Function(name = function_name, description = function_description, parameters = func_parameters, )\n",
    "        user_tool = Tool(function = user_function)\n",
    "        user_tools.append(user_tool)\n",
    "    return user_tools\n"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "id": "e852b86b-2fb8-420f-b186-d96d79d3f038",
   "metadata": {},
   "outputs": [],
   "source": [
    "def bedrock_apicall(prompt):\n",
    "    body = json.dumps({\n",
    "        \"prompt\": prompt,\n",
    "        \"max_tokens\": 512,\n",
    "        \"top_p\": 0.8,\n",
    "        \"temperature\": 0.5,\n",
    "    })\n",
    "    \n",
    "    # modelId = \"mistral.mistral-large-instruct-v0:2\"\n",
    "    modelId = \"mistral.mistral-large-2402-v1:0\"\n",
    "    \n",
    "    accept = \"application/json\"\n",
    "    contentType = \"application/json\"\n",
    "    \n",
    "    response = bedrock.invoke_model(\n",
    "        body=body,\n",
    "        modelId=modelId,\n",
    "        accept=accept,\n",
    "        contentType=contentType\n",
    "    )\n",
    "    \n",
    "    #print(json.loads(response.get('body').read()))\n",
    "    return response.get('body').read()\n",
    "    "
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "id": "c7c54df2-0f27-4ad0-b1d4-bd515afee147",
   "metadata": {},
   "outputs": [],
   "source": [
    "def generate_tool_prompt(queries):\n",
    "    return_objs = [['pet','petId'], ['user', 'username'], ['store/order','orderId']]\n",
    "    function_end_point = 'https://petstore3.swagger.io/api/v3/openapi.json'\n",
    "\n",
    "    user_messages=get_user_messages(queries)\n",
    "    user_tools = generate_tools(return_objs, function_end_point)\n",
    "\n",
    "    tokenizer = MistralTokenizer.v3()\n",
    "    completion_request = ChatCompletionRequest(tools=user_tools, messages=user_messages,)\n",
    "    tokenized = tokenizer.encode_chat_completion(completion_request)\n",
    "    _, text = tokenized.tokens, tokenized.text\n",
    "    prompt=text\n",
    "    return prompt"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "id": "747a2113-73e2-48ba-8a64-1c9cb5018d97",
   "metadata": {},
   "outputs": [],
   "source": [
    "def voice_capture():\n",
    "\n",
    "\n",
    "    # Set audio parameters\n",
    "    CHUNK = 1024\n",
    "    FORMAT = pyaudio.paInt16\n",
    "    CHANNELS = 1\n",
    "    RATE = 16000\n",
    "    RECORD_SECONDS = 7\n",
    "\n",
    "    # Initialize PyAudio\n",
    "    p = pyaudio.PyAudio()\n",
    "\n",
    "    # Open audio stream\n",
    "    stream = p.open(format=FORMAT,\n",
    "                    channels=CHANNELS,\n",
    "                    rate=RATE,\n",
    "                    input=True,\n",
    "                    frames_per_buffer=CHUNK)\n",
    "\n",
    "    print(\"Recording...\")\n",
    "\n",
    "    # Create a list to store audio frames\n",
    "    frames = []\n",
    "\n",
    "    # Record audio for the specified duration\n",
    "    for i in range(0, int(RATE / CHUNK * RECORD_SECONDS)):\n",
    "        data = stream.read(CHUNK)\n",
    "        frames.append(data)\n",
    "\n",
    "    print(\"Finished recording.\")\n",
    "\n",
    "    # Stop and close the audio stream\n",
    "    stream.stop_stream()\n",
    "    stream.close()\n",
    "    p.terminate()\n",
    "\n",
    "    # Save the recorded audio to a WAV file\n",
    "    wf = wave.open(\"output.wav\", 'wb')\n",
    "    wf.setnchannels(CHANNELS)\n",
    "    wf.setsampwidth(p.get_sample_size(FORMAT))\n",
    "    wf.setframerate(RATE)\n",
    "    wf.writeframes(b''.join(frames))\n",
    "    wf.close()\n"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "id": "258a8fdd-8296-4a4c-8c83-78ce6d85c6ed",
   "metadata": {},
   "outputs": [],
   "source": [
    "def getPetById(petId: int) -> str:\n",
    "    try:\n",
    "        method = 'GET'\n",
    "        headers=None\n",
    "        data=None\n",
    "        url =  'https://petstore3.swagger.io/api/v3/pet/' + str(petId)\n",
    "        response = requests.request(method, url, headers=headers, data=data)\n",
    "        # Raise an exception if the response was unsuccessful\n",
    "        response.raise_for_status()\n",
    "        #response = make_api_call('GET', url + str(petId))\n",
    "        if response.ok :\n",
    "            json_response = response.json()\n",
    "            if petId == json_response['id']:\n",
    "                return json_response\n",
    "        return json.dumps({'error': 'Pet id not found.'})\n",
    "    except requests.exceptions.HTTPError as e:\n",
    "        if response.status_code == 404:\n",
    "            return json.dumps({'error': 'Pet id not found.'})\n",
    "        else:\n",
    "            return json.dumps({'error': 'Error with API.'})\n",
    "def getUserByName(username: str) -> str:\n",
    "    try:\n",
    "        url = 'https://petstore3.swagger.io/api/v3/user/' + username\n",
    "        response = requests.get(url)\n",
    "        # Raise an exception if the response was unsuccessful\n",
    "        response.raise_for_status()\n",
    "        if response.ok :\n",
    "            json_response = response.json()\n",
    "            if username == json_response['username']:\n",
    "                return json_response\n",
    "        return json.dumps({'error': 'Username id not found.'})\n",
    "    except requests.exceptions.HTTPError as e:\n",
    "        if response.status_code == 404:\n",
    "            return json.dumps({'error': 'Username not found.'})\n",
    "        else:\n",
    "            return json.dumps({'error': 'Error with API.'})\n",
    "\n",
    "names_to_functions = {\n",
    "  'getPetById': functools.partial(getPetById, petId=''),\n",
    "  'getUserByName': functools.partial(getUserByName, username='')  \n",
    "}"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "id": "54b27857-1e63-4b55-9ded-68f5134c77fb",
   "metadata": {},
   "outputs": [],
   "source": [
    "def process_results(result):\n",
    "    result_format = result['outputs'][0]['text']\n",
    "\n",
    "    result_tool_calls = (result_format.replace(\"[TOOL_CALLS] \",\"\"))\n",
    "\n",
    "    result_tool_calls = eval(result_tool_calls)\n",
    "\n",
    "    function_name = result_tool_calls[0][\"name\"]\n",
    "    function_params = (result_tool_calls[0][\"arguments\"]) \n",
    "    function_result = names_to_functions[function_name](**function_params)\n",
    "    print(function_result)\n"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "id": "79126d76-7327-4d87-bfef-791bac6e6a77",
   "metadata": {},
   "outputs": [],
   "source": [
    "def voice_query():\n",
    "    url = \"http://localhost:5000/whisper\"\n",
    "    #files = {'file': open('/path/to/filename.mp3', 'rb')}\n",
    "    #files = {'file': open('../data/test1.flac', 'rb')}\n",
    "    files = {'file': open('output.wav', 'rb')}\n",
    "    response = requests.post(url, files=files)\n",
    "\n",
    "    response_query = \"\"\n",
    "    if response.status_code == 200:\n",
    "        query = json.loads(response.text)\n",
    "        for result in query['results']:\n",
    "            transcript = result['transcript']\n",
    "            response_query = response_query + transcript\n",
    "        return response_query    \n",
    "    else:\n",
    "        print(f\"Error: {response.status_code} - {response.text}\")"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "id": "ef2beeb9-47d6-4470-a378-6753d847a65c",
   "metadata": {},
   "outputs": [],
   "source": [
    "    voice_capture()\n",
    "    voice_response = voice_query()\n",
    "    print(\"speech recognition output- \" + voice_response)\n",
    "    #queries = [\"What's the status of my Pet 1?\", \"Find information of user user1?\" ,  \"What's the status of my Store Order 3?\"]\n",
    "    queries = []\n",
    "    queries.append(voice_response)\n",
    "    return_objs = [['pet','petId'], ['user', 'username'], ['store/order','orderId']]\n",
    "\n",
    "    prompt_text = generate_tool_prompt(queries)\n",
    "    response = bedrock_apicall(prompt_text)\n",
    "    response_json = json.loads(response)\n",
    "    #print(response_json)\n",
    "    process_results(response_json)\n",
    "    #execute_generator(queries, return_objs)\n",
    "    "
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "id": "8b2db9c4-b8c5-44bf-ad4f-8568ecd1a40c",
   "metadata": {},
   "outputs": [],
   "source": []
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "id": "c6cbeff1-3003-423f-95f2-192a84ec2d66",
   "metadata": {},
   "outputs": [],
   "source": []
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "id": "7949d905-ffb6-4a45-b725-717911144252",
   "metadata": {},
   "outputs": [],
   "source": []
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "id": "7f40089c-880d-4813-8b24-f1e6bf6d0b5c",
   "metadata": {},
   "outputs": [],
   "source": []
  }
 ],
 "metadata": {
  "kernelspec": {
   "display_name": "Python 3 (ipykernel)",
   "language": "python",
   "name": "python3"
  },
  "language_info": {
   "codemirror_mode": {
    "name": "ipython",
    "version": 3
   },
   "file_extension": ".py",
   "mimetype": "text/x-python",
   "name": "python",
   "nbconvert_exporter": "python",
   "pygments_lexer": "ipython3",
   "version": "3.10.12"
  }
 },
 "nbformat": 4,
 "nbformat_minor": 5
}
